/* 
   Copyright (C) 2010 German A. Arias <german@xelalug.org>

   This file is part of Gemas application

   Gemas is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 3 of the License, or (at your option) any later version.
 
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
 
   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#import "GemasDocument.h"
#import <HighlighterKit/HighlighterKit.h>
#import "GemasEditorView.h"

@interface GemasDocument (Private)

- (void) setupNewDefaults;

@end

@implementation GemasDocument (Private)

- (void) setupNewDefaults
{
  NSUserDefaults * df = [NSUserDefaults standardUserDefaults];
  NSData * data;

  ASSIGN (defaultFont, [HKSyntaxHighlighter defaultFont]);

  data = [df dataForKey: @"EditorTextColor"];
  if (data != nil)
    {
      ASSIGN (textColor, [NSKeyedUnarchiver unarchiveObjectWithData: data]);
    }

  data = [df dataForKey: @"EditorBackgroundColor"];
  if (data != nil)
    {
      ASSIGN (backgroundColor, [NSKeyedUnarchiver unarchiveObjectWithData:
        data]);
    }

  data = [df dataForKey: @"EditorInsertionPointColor"];
  if (data != nil)
    {
      ASSIGN (insertionPointColor, [NSKeyedUnarchiver
        unarchiveObjectWithData: data]);
    }


  if (textColor != nil)
    {
      [textView setTextColor: textColor];
    }
  if (backgroundColor != nil)
    {
      [textView setBackgroundColor: backgroundColor];
      [textView setDrawsBackground: YES];
    }
  if (insertionPointColor != nil)
    {
      [textView setInsertionPointColor: insertionPointColor];
    }

  [textView setNeedsDisplay: YES];
}

@end

@implementation GemasDocument

- (void) dealloc
{
  TEST_RELEASE (string);
  TEST_RELEASE (defaultFont);
  TEST_RELEASE (textColor);
  TEST_RELEASE (backgroundColor);
  TEST_RELEASE (insertionPointColor);

  [[NSNotificationCenter defaultCenter] removeObserver: self];

  [super dealloc];
}

- (id)init
{
  if ((self = [super init]) != nil)
    {
      [[NSNotificationCenter defaultCenter]
        addObserver: self
           selector: @selector (defaultsChanged:)
               name: CodeEditorDefaultsDidChangeNotification
             object: nil];

      return self;
    }
  else
    {
      return nil;
    }
}

- (BOOL) readFromFile: (NSString *) fileName ofType: (NSString *) fileType
{
  NSString * aString = [NSString stringWithContentsOfFile: fileName];
  NSString *insert = nil;

  if (aString != nil)
    {
      NSInteger tab = [[NSUserDefaults standardUserDefaults]
			integerForKey: @"Indentation"];

      switch (tab)
	{
	case 0:
	  insert = [NSString stringWithString: @"        "];
	  break;
	case 1:
	  insert = [NSString stringWithString: @"    "];
	  break;
	case 2:
	  insert = [NSString stringWithString: @"  "];
	  break;
	}

      aString = [aString stringByReplacingOccurrencesOfString: @"\t" withString: insert];
      ASSIGN (string, aString);

      return YES;
    }
  else
    {
      return NO;
    }
}

- (BOOL) writeToFile: (NSString *) fileName ofType: (NSString *) fileType
{
  BOOL result;
  result = [[textView string] writeToFile: fileName atomically: NO];

  return result;
}

- (void) awakeFromNib
{
  NSMutableDictionary *typingAttrs;
  
  // don't allow users to change the font with the font panel
  [textView setFont: defaultFont];
  
  /* turn off ligatures - they're useless for a code editor */
  typingAttrs = [[[textView typingAttributes] mutableCopy] autorelease];
  [typingAttrs setObject: [NSNumber numberWithInt: 0]
	       forKey: NSLigatureAttributeName];
  [textView setTypingAttributes: typingAttrs];
  [textView setAllowsUndo: YES];
  [textView setUsesFindPanel: YES];
  
  [textView replaceCharactersInRange: NSMakeRange(0, 0) withString: string];
  [textView setSelectedRange: NSMakeRange(0, 0)];
  DESTROY(string);
  
  //Setup the user preferences
  [self setupNewDefaults];
  
  //Set the syntax highlighter
  [textView createSyntaxHighlighterForFileType: [self fileType]];
}

- (NSString *) windowNibName
{
  return @"Editor";
}

- (void) goToLineNumber: (id) sender
{
  NSUInteger offset = 0;
  NSUInteger i;
  NSUInteger lineNumber;
  NSRange rLine;
  NSString *line;
  NSArray *lines = [[textView string] componentsSeparatedByString: @"\n"];
  NSEnumerator *search = [lines objectEnumerator];


  if ([sender intValue] < 0)
    {
      lineNumber = -1*[sender intValue];
    }
  else
    {
      lineNumber = [sender intValue];
    }

  for (i=1; (line = [search nextObject]) != nil && i < lineNumber;
       i++)
    {
      offset += [line length] + 1;
    }
  
  if (line != nil)
    {
      rLine = NSMakeRange(offset, [line length]);
    }
  else
    {
      rLine = NSMakeRange([[textView string] length], 0);
    }

  [textView setSelectedRange: rLine];
  [textView scrollRangeToVisible: rLine];
  [[sender window] performClose: self];
}

- (void) textViewDidChangeSelection: (NSNotification *) notification
{
  //Number of line and column
  NSRange tex = [textView selectedRange];
  NSArray *selectedLines = [[[textView string] substringWithRange: NSMakeRange(0, tex.location)] componentsSeparatedByString: @"\n"];

  [numberLine setIntValue: [selectedLines count]];
  [numberColumn setIntValue: [[selectedLines lastObject] length] + 1];
}

- (NSColor *) textColor
{
  return textColor;
}

- (void) defaultsChanged: (NSNotification *) notif
{
  [self setupNewDefaults];
}

@end
